;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Coin Game Cloud (sprite 6A), by imamelia
;;
;; This is a disassembly of sprite 6A in SMW, the coin game cloud.
;;
;; Uses first extra bit: NO
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Data1:
db $10,$F0

Data2:
db $01,$FF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR CoinCloudMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine and part of the GFX routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CoinCloudMain:

LDA $C2,x	; if the sprite has just started going...
BNE NoReset	;
INC $C2,x	; increment the sprite state
STZ $18E3	; and set the number of coins collected to 0
NoReset:		; (why couldn't this have just been done in the init routine?)

LDA $9D		; if sprites are locked...
BNE Draw		; skip right to the GFX routine

LDA $14		;
AND #$7F	; unless the frame counter is at 00 or 80...
BNE NoSpawn	; don't spawn anything
LDA $1570,x	;
CMP #$0B		; if the smiley-face coin counter has maxed out...
BCS NoSpawn	; don't spawn anything

INC $1570,x	; increment the counter
JSR Spawn		; spawn a coin (or 1-Up, as the case may be)

NoSpawn:		;

LDA $14		;
AND #$01	; every other frame...
BNE Skip1		; skip some speed setting and stuff

LDA $D8,x	;
STA $00		; note the sprite Y position
LDA $14D4,x	;
STA $01		;
LDA #$10		;
STA $02		; 110 will be the check number
LDA #$01		;
STA $03		;

REP #$20		;
LDA $00		;
CMP $02		; if the sprite Y position is greater than 110...
SEP #$20		;
LDY #$00		; Y = 00
BCC LessThan110	; if less than 110,
INY		; Y = 01
LessThan110:	;

LDA $1570,x	;
CMP #$0B		; if the counter has not maxed out...
BCC Skip2	; don't call the offscreen processing code

JSR SubOffscreenX0	; if the counter has maxed out, check the sprite's offscreen status
LDY #$01		; and set Y = 01

Skip2:		;

LDA $AA,x	; check the sprite's Y speed
CMP Data1,y	; if has reached 10 in either direction...
BEQ Skip1		; don't add to it any more
CLC		;
ADC Data2,y	;
STA $AA,x	;

Skip1:		;

JSL $01801A	; update sprite Y position without gravity

LDA #$08		;
STA $B6,x		; set the sprite's X speed

JSL $018022	; update sprite X position without gravity

Draw:		; built-in GFX routine

LDA $15EA,x	;
PHA		; preserve the OAM index
CLC		;
ADC #$04	; so we can increment it by 4 to access the next slot...
STA $15EA,x	;

JSR DrawSome	; had to code this myself; it was originally a shared GFX routine

PLA		;
STA $15EA,x	; get the original OAM index back

JSR GetDrawInfo	; commented this because I already used it once

LDA $00		;
CLC		;
ADC #$04	; X displacement of 04
STA $0300,y	;

LDA $01		;
CLC		;
ADC #$04	; Y displacement of 04 as well
STA $0301,y	;

LDA #$4D	; smiley-face tile
STA $0302,y	;

LDA #$39		; highest priority, palette C, second GFX page
STA $0303,y	;

LDY #$00		;
LDA #$00		; one 8x8 tile
JSL $81B7B3	;

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; remainder of the GFX routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DrawSome:	; this sprite does its GFX kind of weirdly

JSR GetDrawInfo	;

LDA $00		;
STA $0300,y	;

LDA $01		;
STA $0301,y	; no X or Y displacement to bother with

LDA #$60		; set the cloud tile number
STA $0302,y	;

LDA $14		; frame counter
ASL #3		; x8
AND #$C0	; only the two highest bits, for XY-flipping the cloud tile
ORA #$30		; set the highest priority on the cloud tile
STA $0303,y	;

LDY #$02		;
LDA #$00		; one 16x16 tile
JSL $81B7B3	;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; coin/1-Up-spawning routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Spawn:		;

LDA $18E3	;
CMP #$0A	; if the spawn counter is less than 0A...
BCC SpawnCoin	; spawn a coin, not a 1-Up mushroom

LDY #$0B		; 12 sprite indexes to loop through
Loop1Up:		; odd method of finding a sprite slot...
LDA $14C8,y	;
BEQ Spawn1Up	; if this sprite slot is free, spawn a 1-Up
DEY		;
CPY #$09		; apparently, the 1-Up has to use sprite index 0A or 0B...huh?
BNE Loop1Up	;

Spawn1Up:	;

LDA #$08		;
STA $14C8,y	; normal sprite status

LDA #$78		;
STA $009E,y	; sprite number = 78, 1-Up mushroom

LDA $E4,x	;
STA $00E4,y	;
LDA $14E0,x	;
STA $14E0,y	;
LDA $D8,x	; sprite X and Y position the same as the cloud
STA $00D8,y	;
LDA $14D4,x	;
STA $14D4,y	;

PHX		; preserve the cloud sprite index
TYX		; get the 1-Up sprite index into X
JSL $07F7D2	; initialize sprite tables
LDA #$E0		;
STA $AA,x	; give the 1-Up a little upward speed
INC $157C,x	; and make it face left
PLX		; current sprite index back into X

RTS		;

SpawnCoin:	;

LDA $1570,x	; if the counter has gone up to 0B...
CMP #$0B		; "It goes all the way up to ELEVEN!"
BCS EndSpawn	; don't spawn a coin...or anything else, for that matter

LDY #$07		;
LoopCoin:	;
LDA $170B,y	; find a free extended sprite slot for the smiley-face coin
BEQ CoinSpawnRt	; if it's free, spawn a coin in that slot
DEY		; unlike before, we're using all of the extended sprite slots,
BPL LoopCoin	; so if Y is still positive, keep checking for a free slot

EndSpawn:	;
RTS

CoinSpawnRt:	;

LDA #$0A		; extended sprite number = 0A
STA $170B,y	; coin from coin cloud...well, DUH! Which sprite is this a disassembly of?

LDA $E4,x	;
CLC		;
ADC #$04	; spawn the coin 4 pixels to the right of the cloud
STA $171F,y	;
LDA $14E0,x	;
ADC #$00	; handle the high byte to prevent overflow
STA $1733,y	;

LDA $D8,x	;
STA $1715,y	; same Y position
LDA $14D4,x	;
STA $1729,y	;

LDA #$D0	; give the coin some upward Y speed
STA $173D,y	;

LDA #$00		;
STA $1747,y	; but no X speed...
STA $1765,x	; and...I believe this is a timer of some kind...

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS

